@@ -201,7 +201,7 @@ class Integer(TypedField, Number):
         Number._validate_static(self, value)
 
 
-class DecimalNumber(Number):
+class DecimalNumber(Number, SerializableField):
     """
     An extension of :class:`Number` for a Decimal. Accepts anything that can be converted to a Decimal.
     It converts the value to a Decimal.
@@ -217,6 +217,11 @@ class DecimalNumber(Number):
 
         super().__set__(instance, value)
 
+    def serialize(self, value):
+        return float(value)
+
+
+
 
 class StructureClass(TypedField):
     _ty = StructMeta
@@ -359,7 +364,7 @@ class Positive(Number):
 
     def __set__(self, instance, value):
         if value <= 0:
-            raise ValueError("{}: Must be positive".format(self._name))
+            raise ValueError("{}: Got {}; Expected a positive number".format(self._name, value))
         super().__set__(instance, value)
 
 
@@ -1300,12 +1305,18 @@ class Enum(Field, metaclass=_EnumMeta):
         if self._is_enum:
             enum_names = {v.name for v in self._enum_class}
             if value not in enum_names and not isinstance(value, (self._enum_class,)):
+                enum_values = [r.name for r in self._enum_class]
+                if len(enum_values) < 11:
+                    raise ValueError(
+                        "{}: Got {}; Expected one of: {}".format(self._name, value, ", ".join(enum_values))
+                    )
                 raise ValueError(
-                    "{}: Must be a value of {}".format(self._name, self._enum_class)
+                    "{}: Got {}; Expected a value of {}".format(self._name, value, self._enum_class)
                 )
 
         elif value not in self.values:
-            raise ValueError("{}: Must be one of {}".format(self._name, self.values))
+            raise ValueError("{}: Got {}; Expected one of {}".format(
+                self._name, value, ', '.join([str(v) for v in self.values])))
 
     def __set__(self, instance, value):
         self._validate(value)
@@ -1351,7 +1362,7 @@ class Sized(Field):
 
     def __set__(self, instance, value):
         if len(value) > self.maxlen:
-            raise ValueError("{}: Too long".format(self._name))
+            raise ValueError("{}: Got {}; Expected a length up to {}".format(self._name, value, self.maxlen))
         super().__set__(instance, value)
 
 
