@@ -3,12 +3,14 @@ The Skeleton classes to support strictly defined structures:
 Structure, Field, StructureReference, ClassReference, TypedField
 """
 import enum
+import json
 from copy import deepcopy
 from collections import OrderedDict, deque, defaultdict
 from inspect import Signature, Parameter, signature
 import sys
 import typing
 import hashlib
+import inspect
 
 from typing import get_type_hints, Iterable
 
@@ -48,7 +50,7 @@ class ImmutableMixin:
             raise ValueError("{}: Field is immutable".format(name))
 
 
-def make_signature(names, required, additional_properties, bases_params_by_name):
+def make_signature(names, required, additional_properties, bases_params_by_name, bases_required):
     """
     Make a signature that will be used for the constructor of the Structure
     :param names: names of the properties
@@ -70,7 +72,7 @@ def make_signature(names, required, additional_properties, bases_params_by_name)
         [
             (name, param)
             for (name, param) in bases_params_by_name.items()
-            if name in required
+            if (name in required or name in bases_required)
         ]
     )
     non_default_args = list(
@@ -88,7 +90,7 @@ def make_signature(names, required, additional_properties, bases_params_by_name)
         [
             (name, param)
             for (name, param) in bases_params_by_name.items()
-            if name not in required
+            if (name not in required and name not in bases_required)
         ]
     )
     default_args = list({**default_args_for_bases, **default_args_for_class}.values())
@@ -403,10 +405,10 @@ def _get_all_fields_by_name(cls):
     all_classes = reversed([c for c in cls.mro() if isinstance(c, StructMeta)])
     all_fields_by_name = {}
     for the_class in all_classes:
-        field_by_name = {
-            k: v for k, v in the_class.__dict__.items() if isinstance(v, Field)
-        }
-        all_fields_by_name.update(field_by_name)
+        if issubclass(the_class, Structure):
+            field_names = getattr(the_class, "_fields", [])
+            field_by_name = dict([(k, getattr(the_class, k)) for k in field_names])
+            all_fields_by_name.update(field_by_name)
     return all_fields_by_name
 
 
@@ -490,8 +492,10 @@ class StructMeta(type):
             list(set(bases_required + fields)) if bases_params else fields
         )
         required = cls_dict.get(REQUIRED_FIELDS, default_required)
+        setattr(clsobj, REQUIRED_FIELDS, list(set(bases_required + required)))
         additional_props = cls_dict.get(ADDITIONAL_PROPERTIES, True)
-        sig = make_signature(clsobj._fields, required, additional_props, bases_params)
+        sig = make_signature(clsobj._fields, required, additional_props, bases_params,
+                             bases_required=bases_required)
         setattr(clsobj, "__signature__", sig)
         return clsobj
 
@@ -604,7 +608,8 @@ def add_annotations_to_class_dict(cls_dict):
         for k, v in annotations.items():
             first_arg = getattr(v, "__args__", [0])[0]
             mros = getattr(first_arg, "__mro__", getattr(v, "__mro__", []))
-            if isinstance(v, (Field, Structure)) or Field in mros or Structure in mros:
+            if not _type_is_generic(v) and (
+                    isinstance(v, (Field, Structure)) or Field in mros or Structure in mros):
                 if k in cls_dict:
                     defaults[k] = cls_dict[k]
                 cls_dict[k] = v
@@ -698,6 +703,7 @@ class Structure(UniqueMixin, metaclass=StructMeta):
     """
 
     _fields = []
+    _fail_fast = True
 
     def __init__(self, *args, **kwargs):
         bound = getattr(self, "__signature__").bind(*args, **kwargs)
@@ -705,8 +711,20 @@ class Structure(UniqueMixin, metaclass=StructMeta):
             for name, val in bound.arguments["kwargs"].items():
                 setattr(self, name, val)
             del bound.arguments["kwargs"]
-        for name, val in bound.arguments.items():
-            setattr(self, name, val)
+
+        if Structure.failing_fast():
+            for name, val in bound.arguments.items():
+                setattr(self, name, val)
+        else:
+            errors = []
+            for name, val in bound.arguments.items():
+                try:
+                    setattr(self, name, val)
+                except (TypeError, ValueError) as ex:
+                    errors.append(ex)
+            if errors:
+                messages = json.dumps([str(e) for e in errors])
+                raise errors[0].__class__(messages) from errors[0]
 
         self.__validate__()
         self._instantiated = True
@@ -724,10 +742,13 @@ class Structure(UniqueMixin, metaclass=StructMeta):
             if key in self.__dict__:
                 raise ValueError("Structure is immutable")
             value = deepcopy(value)
-        if all([getattr(self, IGNORE_NONE_VALUES, False),
-                value is None,
-                key not in getattr(self.__class__, REQUIRED_FIELDS, [])]
-               ):
+        if all(
+                [
+                    getattr(self, IGNORE_NONE_VALUES, False),
+                    value is None,
+                    key not in getattr(self.__class__, REQUIRED_FIELDS, []),
+                ]
+        ):
             return
 
         super().__setattr__(key, value)
@@ -857,6 +878,20 @@ class Structure(UniqueMixin, metaclass=StructMeta):
             "{} does not support this operator".format(self.__class__.__name__)
         )
 
+    def shallow_clone_with_overrides(self, **kw):
+        fields_names = self.get_all_fields_by_name().keys()
+        field_value_by_name = dict([(f, getattr(self, f)) for f in fields_names if getattr(self, f) is not None])
+        kw_args = {**field_value_by_name, **kw}
+        return self.__class__(**kw_args)
+
+    @staticmethod
+    def set_fail_fast(fast_fail: bool):
+        Structure._fail_fast = fast_fail
+
+    @staticmethod
+    def failing_fast():
+        return Structure._fail_fast
+
 
 class FinalStructure(Structure):
     pass
